@@ -1,7 +1,8 @@
 from importlib import import_module
+from types import ModuleType
 
 
-def import_string(dotted_path):
+def import_string(dotted_path: str) -> ModuleType:
     try:
         module_path, class_name = dotted_path.rsplit('.', 1)
     except ValueError:
@@ -12,6 +13,8 @@ def import_string(dotted_path):
     try:
         return getattr(module, class_name)
     except AttributeError:
-        raise ImportError('Module "%s" does not define a "%s" attribute/class' % (
-            module_path, class_name)
-                          )
+        raise ImportError(
+            'Module "%s" does not define a "%s" attribute/class' % (
+                module_path, class_name,
+            ),
+        )
