@@ -1,10 +1,11 @@
-import shlex
 import os
 import pwd
+import shlex
 import warnings
+from typing import Tuple
 
 from ..exceptions import B5ExecutionError
-from . import BaseModule
+from . import CONFIG_PREFIX_RE, BaseModule
 
 
 class DockerModule(BaseModule):
@@ -24,9 +25,19 @@ class DockerModule(BaseModule):
         'docker_machine': None,
         'commands': {},
         'sync': {},
+        'setup': {},
     }
 
-    def prepare_config(self):
+    # Valid combinations of compose filenames according to Compose File Specification
+    # See https://docs.docker.com/compose/compose-file/#compose-file
+    COMPOSE_CONFIG_FILENAMES = [
+        ('compose', 'yaml'),
+        ('compose', 'yml'),
+        ('docker-compose', 'yaml'),
+        ('docker-compose', 'yml'),
+    ]
+
+    def prepare_config(self) -> None:  # noqa: C901
         self.config['base_path'] = os.path.realpath(os.path.join(
             self.state.run_path,
             self.config['base_path'],
@@ -45,39 +56,95 @@ class DockerModule(BaseModule):
             raise B5ExecutionError('docker commands has to be a dictionary')
         if not isinstance(self.config['sync'], dict):
             raise B5ExecutionError('docker sync has to be a dictionary')
+        if not isinstance(self.config['setup'], dict):
+            raise B5ExecutionError('setup has to be a dictionary')
+        else:
+            if 'networks' in self.config['setup']:
+                if not isinstance(self.config['setup']['networks'], list):
+                    raise B5ExecutionError('setup.networks has to be a list')
 
-        ##### CONFIGURATION MANAGEMENT #####
+        # ##### CONFIGURATION MANAGEMENT #####
 
-        # make sure docker_compose_configs is a list
+        # Make sure docker_compose_configs is a list
         if self.config['docker_compose_configs'] is not None:
             if not isinstance(self.config['docker_compose_configs'], list):
                 self.config['docker_compose_configs'] = [self.config['docker_compose_configs']]
-        # make sure docker_compose_config_overrides is a list if set
+            for docker_compose_config in self.config['docker_compose_configs']:
+                if docker_compose_config.startswith('docker-compose'):
+                    warnings.warn(f'The compose file {docker_compose_config} should be renamed to '
+                                  f'comply with the newest Compose File Specification. Compose implementations '
+                                  f'are no longer required to support the old docker-compose naming scheme.')
+        # Make sure docker_compose_config_overrides is a list if set
         if self.config['docker_compose_config_overrides'] is not None:
             if not isinstance(self.config['docker_compose_config_overrides'], list):
                 self.config['docker_compose_config_overrides'] = [self.config['docker_compose_config_overrides']]
-        # add docker_compose_config_override to docker_compose_config_overrides if set
+        # Add docker_compose_config_override to docker_compose_config_overrides if set
         # (+ warn user to use docker_compose_config_overrides)
         if self.config['docker_compose_config_override'] is not None:
-            warnings.warn('Use docker_compose_config_overrides instead of docker_compose_config_override (mind the plural form)')
+            warnings.warn('Use docker_compose_config_overrides instead of '
+                          'docker_compose_config_override (mind the plural form)')
             if isinstance(self.config['docker_compose_config_overrides'], list):
-                # self.config['docker_compose_config_overrides'].append(self.config['docker_compose_config_override'])
-                raise B5ExecutionError('You cannot mix docker_compose_config_override and docker_compose_config_overrides, '
-                                       'normally you have to now update your config.local.yml')
+                raise B5ExecutionError('You cannot mix docker_compose_config_override '
+                                       'and docker_compose_config_overrides, normally '
+                                       'you have to now update your config.local.yml')
             else:
                 self.config['docker_compose_config_overrides'] = [self.config['docker_compose_config_override']]
-        # merge docker_compose_config_overrides and docker_compose_configs
-        if isinstance(self.config['docker_compose_config_overrides'], list):
-            docker_compose_configs_was_empty = False
-            if not isinstance(self.config['docker_compose_configs'], list):
-                docker_compose_configs_was_empty = True
-                self.config['docker_compose_configs'] = ['docker-compose.yml']
-            for override in self.config['docker_compose_config_overrides']:
-                self.config['docker_compose_configs'].append('docker-compose.%s.yml' % override)
-            if docker_compose_configs_was_empty and os.path.exists(os.path.join(self.config['base_path'], 'docker-compose.override.yml')):
-                self.config['docker_compose_configs'].append('docker-compose.override.yml')
 
-    def get_script_env(self):
+        # Find correct config file name schema
+        docker_compose_configs_was_empty = True
+        docker_compose_config_prefix = None
+        docker_compose_config_extension = None
+        if isinstance(self.config['docker_compose_configs'], list) and self.config['docker_compose_configs']:
+            # Use filename and extension of FIRST passed filename -> for overrides
+            docker_compose_configs_was_empty = False
+            docker_compose_config_prefix, extension = os.path.splitext(self.config['docker_compose_configs'][0])
+            docker_compose_config_extension = extension[1:]
+        else:
+            # Detect the correct filename and extension by searching for the first existing file
+            for prefix, extension in self.COMPOSE_CONFIG_FILENAMES:
+                if os.path.exists(os.path.join(self.config['base_path'], f'{prefix}.{extension}')):
+                    self.config['docker_compose_configs'] = [f'{prefix}.{extension}']
+                    docker_compose_config_prefix = prefix
+                    docker_compose_config_extension = extension
+                    if prefix == 'docker-compose':
+                        # If compose file with old naming scheme exists, warn the user
+                        warnings.warn(
+                            f'The compose override file {prefix}.{extension} should be '
+                            f'renamed to comply with the newest Compose File Specification. '
+                            f'Compose implementations are no longer required to support the '
+                            f'old docker-compose naming scheme.',
+                            DeprecationWarning,
+                            stacklevel=2,
+                        )
+                    break
+
+        # Abort if we found no config file
+        if docker_compose_config_prefix is None:
+            raise B5ExecutionError('No docker compose config file found in run path. Either provide a '
+                                   'valid docker compose config file '
+                                   '(see https://docs.docker.com/compose/compose-file/#compose-file) or use '
+                                   'the docker_compose_configs config option.')
+
+        # Merge docker_compose_config_overrides and docker_compose_configs
+        if isinstance(self.config['docker_compose_config_overrides'], list):
+            for override in self.config['docker_compose_config_overrides']:
+                self.config['docker_compose_configs'].append(f'{docker_compose_config_prefix}'
+                                                             f'.{override}'
+                                                             f'.{docker_compose_config_extension}')
+
+        # Add default override file, if it exists
+        if (
+            docker_compose_configs_was_empty
+            and os.path.exists(os.path.join(
+                self.config['base_path'],
+                f'{docker_compose_config_prefix}.override.{docker_compose_config_extension}',
+            ))
+        ):
+            self.config['docker_compose_configs'].append(f'{docker_compose_config_prefix}'
+                                                         f'.override'
+                                                         f'.{docker_compose_config_extension}')
+
+    def get_script_env(self) -> str:
         import os
 
         params = {
@@ -107,7 +174,14 @@ class DockerModule(BaseModule):
             {docker_machine_env}
         '''.format(**params)
 
-    def _docker_volume_path_str(self, volume_path_name):
+    def is_installed_script(self) -> str:
+        """
+        Add a check to evaluate whether the pipenv module bin is installed or not
+        Returns: str
+        """
+        return self.create_is_installed_script(module=self.name, module_bin=self.config['docker_bin'])
+
+    def _docker_volume_path_str(self, volume_path_name: str) -> Tuple[str, str]:
         # this should be a path
         if '.' in volume_path_name or '/' in volume_path_name:
             return os.path.join(self.config['base_path'], volume_path_name), 'path'
@@ -120,14 +194,14 @@ class DockerModule(BaseModule):
             else:
                 return '_'.join([self.config['project_name'], volume_path_name]), 'volume'
 
-    def get_script(self):
+    def get_script(self) -> str:  # noqa: C901
         script = [super(DockerModule, self).get_script()]
 
         script.append(self._script_config_vars())
 
         script.append(self._script_function_source('install', '''
-            {name}:docker-compose pull
-            {name}:docker-compose build --pull
+            {name}:compose pull
+            {name}:compose build --pull
         '''.format(
             name=self.name,
         )))
@@ -140,13 +214,14 @@ class DockerModule(BaseModule):
         )))
 
         script.append(self._script_function_source('run', '''
+            {name}:setup
             (
                 {docker_env}
                 "$@"
             )
         '''.format(
+            name=self.name,
             docker_env=self.get_script_env(),
-            base_path=shlex.quote(self.config['base_path']),
         )))
 
         script.append(self._script_function_source('docker', '''
@@ -160,7 +235,26 @@ class DockerModule(BaseModule):
             docker_bin=shlex.quote(self.config['docker_bin']),
         )))
 
+        script.append(self._script_function_source('compose', '''
+            (
+                cd {base_path} && \\
+                {name}:run {docker_bin} compose {docker_compose_configs} "$@"
+            )
+        '''.format(
+            base_path=shlex.quote(self.config['base_path']),
+            name=self.name,
+            docker_bin=shlex.quote(self.config['docker_bin']),
+            docker_compose_configs=(
+                '-f %s' % ' -f '.join(map(shlex.quote, self.config['docker_compose_configs']))
+                if self.config['docker_compose_configs']
+                else ''
+            ),
+        )))
+
         script.append(self._script_function_source('docker-compose', '''
+            b5:warn "You are using the old docker-compose v1 callable, you should switch to use '{name}:compose'"
+            b5:warn "in your Taskfile to start using docker compose v2. Note that all internal commands"
+            b5:warn "like '{name}:update' already use '{name}:compose'."
             (
                 cd {base_path} && \\
                 {name}:run {docker_compose_bin} {docker_compose_configs} "$@"
@@ -169,8 +263,11 @@ class DockerModule(BaseModule):
             base_path=shlex.quote(self.config['base_path']),
             name=self.name,
             docker_compose_bin=shlex.quote(self.config['docker_compose_bin']),
-            docker_compose_configs='-f %s' % ' -f '.join(map(shlex.quote, self.config['docker_compose_configs'])) \
-                                    if self.config['docker_compose_configs'] else '',
+            docker_compose_configs=(
+                '-f %s' % ' -f '.join(map(shlex.quote, self.config['docker_compose_configs']))
+                if self.config['docker_compose_configs']
+                else ''
+            ),
         )))
 
         script.append(self._script_function_source('docker-machine', '''
@@ -186,25 +283,39 @@ class DockerModule(BaseModule):
 
         script.append(self._script_function_source('container_id', '''
             (
-                cd {base_path} && \\
-                {name}:docker-compose ps -q "$@" | awk 'NR == 1'
+                cd {base_path} || return 1
+
+                local CONTAINER_ID="$(
+                    {name}:docker ps -q \
+                        --filter label=com.docker.compose.project={project_name} \
+                        --filter label=com.docker.compose.service="$1" \
+                    | head -n 1
+                )"
+
+                if [ -z "$CONTAINER_ID" ]
+                then
+                    return 1
+                fi
+
+                echo "$CONTAINER_ID"
             )
         '''.format(
             base_path=shlex.quote(self.config['base_path']),
+            project_name=shlex.quote(self.config['project_name']),
             name=self.name,
         )))
 
         script.append(self._script_function_source('is_running', '''
             (
                 cd {base_path} || return 1
-                local CONTAINER=$( {name}:container_id "$@" )
 
-                if [ -z $CONTAINER ]
+                local CONTAINER_ID=$( {name}:container_id "$1" || true )
+                if [ -z "$CONTAINER_ID" ]
                 then
                     return 1
                 fi
 
-                if $( docker inspect -f {{{{.State.Running}}}} "$CONTAINER" )
+                if $( docker inspect -f {{{{.State.Running}}}} "$CONTAINER_ID" )
                 then
                     return 0
                 else
@@ -214,23 +325,14 @@ class DockerModule(BaseModule):
         '''.format(
             base_path=shlex.quote(self.config['base_path']),
             name=self.name,
-            docker_machine_bin=shlex.quote(self.config['docker_machine_bin']),
         )))
 
-        # See https://github.com/docker/compose/issues/3352 for why we are not using docker-compose exec here,
-        # this basically is broken on many levels. See for example this comment:
-        # https://github.com/docker/compose/issues/3352#issuecomment-299868032
         script.append(self._script_function_source('container_run', '''
             # docker-compose options
             local options=()
             local exec_options=()
             local run_options=("--rm")
             local use_tty=1
-            # raw docker options (prefix "d_")
-            local d_options=()
-            local d_exec_options=()
-            local d_run_options=("--rm")
-            local d_use_tty=1
 
             local has_valid_options=1
             local force_run=0
@@ -250,19 +352,16 @@ class DockerModule(BaseModule):
                         shift
                         ;;
                     --pipe-in)
-                        d_options+=("-i")
-                        d_use_tty=0
+                        use_tty=0
                         shift
                         ;;
                     --pipe-out)
-                        d_use_tty=0
                         use_tty=0
                         shift
                         ;;
                     # Generic options
                     --disable-tty|-T)
                         use_tty=0
-                        d_use_tty=0
                         shift
                         ;;
                     -u|--user)
@@ -272,8 +371,6 @@ class DockerModule(BaseModule):
                         fi
                         options+=("-u")
                         options+=("$2")
-                        d_options+=("-u")
-                        d_options+=("$2")
                         shift
                         shift
                         ;;
@@ -284,8 +381,6 @@ class DockerModule(BaseModule):
                         fi
                         options+=("-e")
                         options+=("$2")
-                        d_options+=("-e")
-                        d_options+=("$2")
                         shift
                         shift
                         ;;
@@ -296,8 +391,6 @@ class DockerModule(BaseModule):
                         fi
                         options+=("-w")
                         options+=("$2")
-                        d_options+=("-w")
-                        d_options+=("$2")
                         shift
                         shift
                         ;;
@@ -314,8 +407,6 @@ class DockerModule(BaseModule):
                         fi
                         options+=("-l")
                         options+=("$2")
-                        d_options+=("-l")
-                        d_options+=("$2")
                         force_run=1
                         shift
                         shift
@@ -364,20 +455,15 @@ class DockerModule(BaseModule):
             then
                 options+=("-T")
             fi
-            if [ $d_use_tty -gt 0 ]
-            then
-                d_options+=("-it")
-            fi
 
             (
                 cd {base_path} || return 1
 
                 if [ $command_strategy == 'exec' ]
                 then
-                    local container_id=$( {name}:container_id "$container" | head -n 1 )
-                    {name}:docker exec "${{d_options[@]}}" "${{d_exec_options[@]}}" "$container_id" "$@"
+                    {name}:compose exec "${{options[@]}}" "${{exec_options[@]}}" "$container" "$@"
                 else
-                    {name}:docker-compose run "${{options[@]}}" "${{run_options[@]}}" "$container" "$@"
+                    {name}:compose run "${{options[@]}}" "${{run_options[@]}}" "$container" "$@"
                 fi
             )
         '''.format(
@@ -390,7 +476,7 @@ class DockerModule(BaseModule):
                 command_options = {'bin': command, 'service': command}
             elif isinstance(command_options, str):
                 command_options = {'bin': command, 'service': command_options}
-            if not 'bin' in command_options or not 'service' in command_options:
+            if 'bin' not in command_options or 'service' not in command_options:
                 raise B5ExecutionError('You have to specify at least "bin" and "service"')
 
             script.append(self._script_function_source('command:{command}'.format(command=command), '''
@@ -434,19 +520,27 @@ class DockerModule(BaseModule):
                         label=shlex.quote('{key}={value}'.format(
                             key=key,
                             value=value,
-                        ))
+                        )),
                     )
                     for key, value
                     in command_options['labels'].items()
                 ]) if command_options.get('labels') else '',
-                workdir='--workdir {workdir}'.format(workdir=command_options.get('workdir')) if command_options.get('workdir') else '',
-                user='--user {user}'.format(user=command_options.get('user')) if command_options.get('user') else '',
+                workdir=(
+                    '--workdir {workdir}'.format(workdir=command_options.get('workdir'))
+                    if command_options.get('workdir')
+                    else ''
+                ),
+                user=(
+                    '--user {user}'.format(user=command_options.get('user'))
+                    if command_options.get('user')
+                    else ''
+                ),
                 environment=' '.join([
                     '--env {environment}'.format(
                         environment=shlex.quote('{key}={value}'.format(
                             key=key,
                             value=value,
-                        ))
+                        )),
                     )
                     for key, value
                     in command_options['environment'].items()
@@ -457,10 +551,76 @@ class DockerModule(BaseModule):
                     else shlex.quote(command_options.get('bin')),
             )))
 
+        if self.config['setup']:
+            # Setup networks
+            if 'networks' in self.config['setup'] and self.config['setup']['networks']:
+                # Create a function for each network
+                for setup_network_name in self.config['setup']['networks']:
+                    script.append(
+                        self._script_function_source(
+                            'setup:network:{network}'.format(network=setup_network_name), '''
+                                {name}:docker network inspect {network} &>/dev/null || (
+                                    {name}:docker network create {network} && echo \"Created network {network}.\"
+                                );
+                            '''.format(
+                                name=self.name,
+                                network=shlex.quote(setup_network_name),
+                            ),
+                        ),
+                    )
+                # Create general network setup function
+                script.append(self._script_function_source('setup:network', '''
+                    {setup_all_networks}
+                '''.format(
+                    setup_all_networks='\n                    '.join([
+                        '{name}:setup:network:{network}'.format(
+                            name=self.name,
+                            network=setup_network_name,
+                        )
+                        for setup_network_name
+                        in self.config['setup']['networks']
+                    ]),
+                )))
+            # Create general setup function
+            setup_ran_var_name = '_{prefix}_SETUP_RAN'.format(
+                prefix=CONFIG_PREFIX_RE.sub('_', self.name.upper()),
+            )
+            script.append('{setup_ran_var}=0'.format(setup_ran_var=setup_ran_var_name))
+            setup_running_var_name = '_{prefix}_SETUP_RUNNING'.format(
+                prefix=CONFIG_PREFIX_RE.sub('_', self.name.upper()),
+            )
+            script.append('{setup_running_var_name}=0'.format(setup_running_var_name=setup_running_var_name))
+            script.append(self._script_function_source('setup', '''
+                if [ "${setup_ran_var_name}" -eq 1 ]
+                then
+                    return 0
+                fi
+                if [ "${setup_running_var_name}" -eq 1 ]
+                then
+                    return 0
+                fi
+                {setup_running_var_name}=1
+                {setup_networks}
+                {setup_ran_var_name}=1
+            '''.format(
+                setup_networks=(
+                    '{name}:setup:network'.format(name=self.name)
+                    if 'networks' in self.config['setup'] and self.config['setup']['networks']
+                    else ''
+                ),
+                setup_ran_var_name=setup_ran_var_name,
+                setup_running_var_name=setup_running_var_name,
+            )))
+        else:
+            # Create EMPTY general setup function, if no config exists
+            script.append(self._script_function_source('setup', '''
+                true
+            '''))
+
         for sync, sync_options in self.config['sync'].items():
             if not isinstance(sync_options, dict):
                 raise B5ExecutionError('sync options have to contain some options (at least "from" and "to")')
-            if not 'from' in sync_options or not 'to' in sync_options:
+            if 'from' not in sync_options or 'to' not in sync_options:
                 raise B5ExecutionError('You have to specify at least "from" and "to"')
 
             volume_path_from, volume_path_from_type = self._docker_volume_path_str(sync_options['from'])
@@ -491,11 +651,15 @@ class DockerModule(BaseModule):
                 then
                     SYNC_SUBPATH="$1"
                 fi
-            
+
                 # create volumes if it does not exist
                 if [ {volume_path_from_type} == "volume" ]
                 then
-                    local VOLUME_FROM_EXISTS="$( docker volume ls | awk '{{print $2}}' | grep -Fxc {volume_path_from} || true )"
+                    local VOLUME_FROM_EXISTS="$(
+                        docker volume ls \
+                        | awk '{{print $2}}' \
+                        | grep -Fxc {volume_path_from} || true
+                    )"
                     if [ "$VOLUME_FROM_EXISTS" -eq 0 ]
                     then
                         docker volume create {volume_path_from} > /dev/null
@@ -503,7 +667,11 @@ class DockerModule(BaseModule):
                 fi
                 if [ {volume_path_to_type} == "volume" ]
                 then
-                    local VOLUME_TO_EXISTS="$( docker volume ls | awk '{{print $2}}' | grep -Fxc {volume_path_to} || true )"
+                    local VOLUME_TO_EXISTS="$(
+                        docker volume ls \
+                        | awk '{{print $2}}' \
+                        | grep -Fxc {volume_path_to} || true
+                    )"
                     if [ "$VOLUME_TO_EXISTS" -eq 0 ]
                     then
                         docker volume create {volume_path_to} > /dev/null
@@ -524,9 +692,11 @@ class DockerModule(BaseModule):
                 volume_path_from_type=shlex.quote(volume_path_from_type),
                 volume_path_to=shlex.quote(volume_path_to),
                 volume_path_to_type=shlex.quote(volume_path_to_type),
-                docker_image=shlex.quote(sync_options['image']) \
-                    if ('image' in sync_options and sync_options['image']) \
-                    else 'instrumentisto/rsync-ssh:latest',
+                docker_image=(
+                    shlex.quote(sync_options['image'])
+                    if ('image' in sync_options and sync_options['image'])
+                    else 'instrumentisto/rsync-ssh:latest'
+                ),
                 rsync_options=' '.join(rsync_options),
             )))
 
@@ -534,14 +704,16 @@ class DockerModule(BaseModule):
             {syncs}
         '''.format(
             syncs='\n'.join(
-                [shlex.quote('{name}:sync:{sync}'.format(
-                    name=self.name,
-                    sync=sync,
-                ))
-                for sync, sync_options
-                in self.config['sync'].items()
-                if sync_options.get('auto', True)
-            ]) or 'true',
+                [
+                    shlex.quote('{name}:sync:{sync}'.format(
+                        name=self.name,
+                        sync=sync,
+                    ))
+                    for sync, sync_options
+                    in self.config['sync'].items()
+                    if sync_options.get('auto', True)
+                ],
+            ) or 'true',
         )))
 
         return '\n'.join(script)
