@@ -1,5 +1,5 @@
-import shlex
 import os
+import shlex
 
 from . import BaseModule
 
@@ -13,13 +13,20 @@ class NpmModule(BaseModule):
         'npm_bin': 'npm',
     }
 
-    def prepare_config(self):
+    def prepare_config(self) -> None:
         self.config['base_path'] = os.path.realpath(os.path.join(
             self.state.run_path,
             self.config['base_path'],
         ))
 
-    def get_script(self):
+    def is_installed_script(self) -> str:
+        """
+        Add a check to evaluate whether the npm module bin is installed or not
+        Returns: str
+        """
+        return self.create_is_installed_script(module=self.name, module_bin=self.config['npm_bin'])
+
+    def get_script(self) -> str:
         script = [super(NpmModule, self).get_script()]
 
         script.append(self._script_config_vars())
@@ -42,7 +49,7 @@ class NpmModule(BaseModule):
                 "$@"
             )
         '''.format(
-            base_path=shlex.quote(self.config['base_path'])
+            base_path=shlex.quote(self.config['base_path']),
         )))
 
         script.append(self._script_function_source('npm', '''
